package com.zc.security.core.authentication.sms;

import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.InternalAuthenticationServiceException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 短信登录校验器 . <br>
 * <p>
 * <p>
 * Copyright: Copyright (c) 2017/09/27 下午4:57
 * <p>
 * Company: 百趣
 * <p>
 *
 * @author zhongcheng_m@yeah.net
 * @version 1.0.0
 */
public class SmsCodeAuthenticationProvider implements AuthenticationProvider {


    private SmsCodeUserDetailsService smsCodeUserDetailsService;


    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {

        Object userDetail = authentication.getDetails();

        System.out.println(userDetail);


        UserDetails userDetails = smsCodeUserDetailsService.loadUserByMobile((String) authentication.getPrincipal());

        if (userDetails == null) {
            throw new InternalAuthenticationServiceException("无法获取用户信息");
        }

        return createSuccessAuthentication(authentication, userDetails);

    }

    @Override
    public boolean supports(Class<?> authentication) {
        return SmsCodeAuthenticationToken.class.isAssignableFrom(authentication);
    }


    protected Authentication createSuccessAuthentication(
            Authentication authentication, UserDetails user) {
        // Ensure we return the original credentials the user supplied,
        // so subsequent attempts are successful even with encoded passwords.
        // Also ensure we return the original getDetails(), so that future
        // authentication events after cache expiry contain the details
        String principal = user.getUsername();

        SmsCodeAuthenticationToken result = new SmsCodeAuthenticationToken(
                principal,
                user.getAuthorities());
        result.setDetails(authentication.getDetails());
        return result;
    }

    public void setSmsCodeUserDetailsService(SmsCodeUserDetailsService smsCodeUserDetailsService) {
        this.smsCodeUserDetailsService = smsCodeUserDetailsService;
    }
}
